import os
import pytest
from shutil import copy2
import platform

GEOPANDAS_NOT_AVAIL = False
try:
    import geopandas
except ImportError:
    GEOPANDAS_NOT_AVAIL = True

RTREE_NOT_AVAIL = False
try:
    import rtree
except ImportError:
    RTREE_NOT_AVAIL = True

SHAPELY_NOT_AVAIL = False
try:
    import shapely
except ImportError:
    SHAPELY_NOT_AVAIL = True

on_windows = platform.system() == "Windows"

DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
VECTORUTILS_DATA_DIR = os.path.join(DATA_DIR, "vectorutils")
CLASSIFICATION_DATA_DIR = os.path.join(DATA_DIR, "classification")
IMGUTILS_DATA_DIR = os.path.join(DATA_DIR, "imageutils")
REGRESS_DATA_DIR = os.path.join(DATA_DIR, "regression")

# TODO - workout what is happening on Windows.
@pytest.mark.skipif(on_windows, reason="Problems on Windows removing files")
def test_delete_vector_file(tmp_path):
    import rsgislib.vectorutils
    import shutil

    ref_vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    cp_vec_file = os.path.join(tmp_path, "cls_forest_smpls.gpkg")
    shutil.copyfile(ref_vec_file, cp_vec_file)
    if not os.path.exists(cp_vec_file):
        raise Exception("Test file is not present")
    rsgislib.vectorutils.delete_vector_file(cp_vec_file)

    assert not os.path.exists(cp_vec_file)


def test_check_format_name_geojson1():
    import rsgislib.vectorutils

    assert "GeoJSON" == rsgislib.vectorutils.check_format_name("GEOJSON")


def test_check_format_name_geojson2():
    import rsgislib.vectorutils

    assert "GeoJSON" == rsgislib.vectorutils.check_format_name("geojson")


def test_check_format_name_geojson3():
    import rsgislib.vectorutils

    assert "GeoJSON" == rsgislib.vectorutils.check_format_name("GeoJSON")


def test_check_format_name_gpkg():
    import rsgislib.vectorutils

    assert "GPKG" == rsgislib.vectorutils.check_format_name("GPKG")


def test_check_format_name_shp():
    import rsgislib.vectorutils

    assert "ESRI Shapefile" == rsgislib.vectorutils.check_format_name("ESRI Shapefile")


def test_get_proj_wkt_from_vec_osgb():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"
    wkt_str = rsgislib.vectorutils.get_proj_wkt_from_vec(vec_file, vec_lyr)
    assert "British National Grid" in wkt_str


def test_get_proj_epsg_from_vec_osgb():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"
    epsg_code = rsgislib.vectorutils.get_proj_epsg_from_vec(vec_file, vec_lyr)
    assert epsg_code == 27700


def test_get_vec_feat_count():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"
    n_feats = rsgislib.vectorutils.get_vec_feat_count(vec_file, vec_lyr)
    assert n_feats == 9


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_count_feats_per_att_val():
    import rsgislib.vectorutils

    vec_file = os.path.join(
        VECTORUTILS_DATA_DIR, "sen2_20210527_aber_att_vals_dups.geojson"
    )
    vec_lyr = "sen2_20210527_aber_att_vals_dups"
    out_dict = rsgislib.vectorutils.count_feats_per_att_val(vec_file, vec_lyr, "StrCol")
    assert (
        (out_dict["One"] == 2) and (out_dict["Two"] == 2) and (out_dict["Three"] == 2)
    )


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_count_feats_per_att_val_df_dict():
    import rsgislib.vectorutils

    vec_file = os.path.join(
        VECTORUTILS_DATA_DIR, "sen2_20210527_aber_att_vals_dups.geojson"
    )
    vec_lyr = "sen2_20210527_aber_att_vals_dups"
    out_dict = rsgislib.vectorutils.count_feats_per_att_val(
        vec_file, vec_lyr, "StrCol", True
    )
    assert (
        (out_dict["count"][0] == 2)
        and (out_dict["count"][1] == 2)
        and (out_dict["count"][2] == 2)
    )


def test_merge_vectors_to_gpkg(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = [vec_file_1, vec_file_2, vec_file_3, vec_file_4]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.merge_vectors_to_gpkg(in_vec_files, out_vec_file, out_vec_lyr)
    assert os.path.exists(out_vec_file)


def test_merge_vector_lyrs_to_gpkg(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(CLASSIFICATION_DATA_DIR, "cls_poly_smpls.gpkg")

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.merge_vector_lyrs_to_gpkg(vec_file, out_vec_file, out_vec_lyr)
    assert os.path.exists(out_vec_file)


def test_merge_vectors_to_gpkg_ind_lyrs(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = [vec_file_1, vec_file_2, vec_file_3, vec_file_4]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    rsgislib.vectorutils.merge_vectors_to_gpkg_ind_lyrs(in_vec_files, out_vec_file)

    assert os.path.exists(out_vec_file)


def test_merge_vectors_to_gpkg_ind_lyrs_geom_type(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = [vec_file_1, vec_file_2, vec_file_3, vec_file_4]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    rsgislib.vectorutils.merge_vectors_to_gpkg_ind_lyrs(
        in_vec_files, out_vec_file, geom_type="POLYGON"
    )

    assert os.path.exists(out_vec_file)


def test_merge_vectors_to_gpkg_ind_lyrs_geom_type_excp(tmp_path):
    import rsgislib
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = [vec_file_1, vec_file_2, vec_file_3, vec_file_4]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    with pytest.raises(rsgislib.RSGISPyException):
        rsgislib.vectorutils.merge_vectors_to_gpkg_ind_lyrs(
            in_vec_files, out_vec_file, geom_type="ERR"
        )


def test_merge_to_multi_layer_vec(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    input_file_lyrs = list()
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_1, "cls_forest_smpls", "cls_forest_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_2, "cls_grass_smpls", "cls_grass_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_3, "cls_urban_smpls", "cls_urban_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_4, "cls_water_smpls", "cls_water_smpls"
        )
    )

    out_vec_file = os.path.join(tmp_path, "cls_poly_smpls.gpkg")
    rsgislib.vectorutils.merge_to_multi_layer_vec(
        input_file_lyrs, out_vec_file, out_format="GPKG", overwrite=False
    )

    assert os.path.exists(out_vec_file)


def test_merge_to_multi_layer_vec_exists(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    input_file_lyrs = list()
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_1, "cls_forest_smpls", "cls_forest_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_2, "cls_grass_smpls", "cls_grass_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_3, "cls_urban_smpls", "cls_urban_smpls"
        )
    )
    input_file_lyrs.append(
        rsgislib.vectorutils.VecLayersInfoObj(
            vec_file_4, "cls_water_smpls", "cls_water_smpls"
        )
    )

    out_vec_file = os.path.join(tmp_path, "cls_poly_smpls.gpkg")
    rsgislib.vectorutils.merge_to_multi_layer_vec(
        input_file_lyrs, out_vec_file, out_format="GPKG", overwrite=False
    )
    # Run it again so that file exists.
    rsgislib.vectorutils.merge_to_multi_layer_vec(
        input_file_lyrs, out_vec_file, out_format="GPKG", overwrite=True
    )

    assert os.path.exists(out_vec_file)


def test_get_vec_lyrs_lst():
    import rsgislib.vectorutils

    vec_file = os.path.join(CLASSIFICATION_DATA_DIR, "cls_poly_smpls.gpkg")
    lyrs = rsgislib.vectorutils.get_vec_lyrs_lst(vec_file)
    assert len(lyrs) == 4


def test_get_vec_layer_extent():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"
    bbox = rsgislib.vectorutils.get_vec_layer_extent(vec_file, vec_lyr)
    assert (
        (abs(bbox[0] - 259022.6875) < 1)
        and (abs(bbox[1] - 266288.4375) < 1)
        and (abs(bbox[2] - 277206.96875) < 1)
        and (abs(bbox[3] - 282311.96875) < 1)
    )


def test_get_vec_layer_extent_file_excp():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "err_file.gpkg")
    vec_lyr = "err_lyr"
    with pytest.raises(rsgislib.RSGISPyException):
        rsgislib.vectorutils.get_vec_layer_extent(vec_file, vec_lyr)


def test_get_vec_layer_extent_lyr_excp():
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "err_lyr"
    with pytest.raises(rsgislib.RSGISPyException):
        rsgislib.vectorutils.get_vec_layer_extent(vec_file, vec_lyr)


def test_split_vec_lyr(tmp_path):
    import rsgislib.vectorutils
    import glob

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"

    rsgislib.vectorutils.split_vec_lyr(
        vec_file, vec_lyr, 2, "GPKG", tmp_path, "out_vec", "gpkg"
    )

    out_files = glob.glob(os.path.join(tmp_path, "out_vec*.gpkg"))
    assert len(out_files) == 5


def test_reproj_vector_layer(tmp_path):
    import rsgislib.vectorutils
    import rsgislib.tools.utils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    wkt_file = os.path.join(IMGUTILS_DATA_DIR, "utm30n.wkt")
    out_proj_wkt = rsgislib.tools.utils.read_text_file_no_new_lines(wkt_file)

    rsgislib.vectorutils.reproj_vector_layer(
        vec_file,
        out_vec_file,
        out_proj_wkt,
        out_format="GPKG",
        out_vec_lyr=out_vec_lyr,
        vec_lyr=vec_lyr,
        in_proj_wkt=None,
        del_exist_vec=False,
    )
    assert os.path.exists(out_vec_file)


def test_reproj_vec_lyr_obj_4326(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)

    rsgislib.vectorutils.reproj_vec_lyr_obj(
        vec_lyr_obj, out_vec_file, 4326, out_format="GPKG", out_vec_lyr=out_vec_lyr
    )
    vec_obj_ds = None
    assert os.path.exists(out_vec_file)


def test_reproj_vec_lyr_obj_32630(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_lyr = "cls_forest_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)

    rsgislib.vectorutils.reproj_vec_lyr_obj(
        vec_lyr_obj, out_vec_file, 32630, out_format="GPKG", out_vec_lyr=out_vec_lyr
    )
    vec_obj_ds = None
    assert os.path.exists(out_vec_file)


def test_get_att_lst_select_feats():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_sel_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_sel_lyr = "cls_grass_smpls"

    att_names = ["re_b4", "re_b5"]

    out_vals = rsgislib.vectorutils.get_att_lst_select_feats(
        vec_file, vec_lyr, att_names, vec_sel_file, vec_sel_lyr
    )

    assert len(out_vals) == 3


def test_get_att_lst_select_feats_lyr_objs():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"
    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)

    vec_sel_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_sel_lyr = "cls_grass_smpls"
    vec_sel_obj_ds, vec_lyr_sel_obj = rsgislib.vectorutils.open_gdal_vec_lyr(
        vec_sel_file, vec_sel_lyr
    )

    att_names = ["re_b4", "re_b5"]
    out_vals = rsgislib.vectorutils.get_att_lst_select_feats_lyr_objs(
        vec_lyr_obj, att_names, vec_lyr_sel_obj
    )
    vec_obj_ds = None
    vec_sel_obj_ds = None

    assert len(out_vals) == 3


def test_get_att_lst_select_bbox_feats():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    bbox = [262987, 264883, 280213, 282551]

    att_names = ["re_b4", "re_b5"]
    out_vals = rsgislib.vectorutils.get_att_lst_select_bbox_feats(
        vec_file, vec_lyr, att_names, bbox, 27700
    )

    assert len(out_vals) == 7


def test_get_att_lst_select_bbox_feats_lyr_objs():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"
    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)

    bbox = [262987, 264883, 280213, 282551]

    att_names = ["re_b4", "re_b5"]
    out_vals = rsgislib.vectorutils.get_att_lst_select_bbox_feats_lyr_objs(
        vec_lyr_obj, att_names, bbox, 27700
    )
    vec_obj_ds = None
    vec_sel_obj_ds = None

    assert len(out_vals) == 7


def test_select_intersect_feats(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_roi_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_roi_lyr = "cls_grass_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.select_intersect_feats(
        vec_file,
        vec_lyr,
        vec_roi_file,
        vec_roi_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
    )

    assert os.path.exists(out_vec_file)


def test_export_spatial_select_feats(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_roi_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_roi_lyr = "cls_grass_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.export_spatial_select_feats(
        vec_file,
        vec_lyr,
        vec_roi_file,
        vec_roi_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
    )

    assert os.path.exists(out_vec_file)


def test_get_vec_lyr_cols():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    cols = rsgislib.vectorutils.get_vec_lyr_cols(vec_file, vec_lyr)

    assert len(cols) == 6


def test_subset_envs_vec_lyr_obj():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"
    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)

    bbox = [262987, 264883, 280213, 282551]

    vec_rsl_obj_ds, vec_lyr_rsl_obj = rsgislib.vectorutils.subset_envs_vec_lyr_obj(
        vec_lyr_obj, bbox, epsg=27700
    )
    vec_rsl_obj_ds = None


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_subset_veclyr_to_featboxs(tmp_path):
    import rsgislib.vectorutils
    import glob

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_roi_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_roi_lyr = "cls_grass_smpls"

    out_file_base = os.path.join(tmp_path, "out_vec")

    rsgislib.vectorutils.subset_veclyr_to_featboxs(
        vec_roi_file,
        vec_roi_lyr,
        vec_file,
        vec_lyr,
        "sub_grasslands",
        out_file_base,
        out_file_end="gpkg",
        out_format="GPKG",
    )

    out_vecs = glob.glob(os.path.join(tmp_path, "out_vec*.gpkg"))

    assert len(out_vecs) == 11


def test_read_vec_lyr_to_mem():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.read_vec_lyr_to_mem(
        vec_file, vec_lyr
    )
    vec_obj_ds = None


def test_open_gdal_vec_lyr():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.open_gdal_vec_lyr(vec_file, vec_lyr)
    vec_obj_ds = None


def test_get_mem_vec_lyr_subset():
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    bbox = [262987, 264883, 280213, 282551]

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.get_mem_vec_lyr_subset(
        vec_file, vec_lyr, bbox
    )
    vec_obj_ds = None


def test_write_vec_lyr_to_file(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    bbox = [262987, 264883, 280213, 282551]

    vec_obj_ds, vec_lyr_obj = rsgislib.vectorutils.get_mem_vec_lyr_subset(
        vec_file, vec_lyr, bbox
    )

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.write_vec_lyr_to_file(
        vec_lyr_obj, out_vec_file, out_vec_lyr, out_format="GPKG"
    )
    vec_obj_ds = None

    assert os.path.exists(out_vec_file)


def test_create_copy_vector_lyr(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.create_copy_vector_lyr(
        vec_file, vec_lyr, out_vec_file, out_vec_lyr, out_format="GPKG"
    )

    assert os.path.exists(out_vec_file)


def test_get_ogr_vec_col_datatype_from_gdal_rat_col_datatype_int():
    import rsgislib.vectorutils
    from osgeo import gdal
    from osgeo import ogr

    assert (
        rsgislib.vectorutils.get_ogr_vec_col_datatype_from_gdal_rat_col_datatype(
            gdal.GFT_Integer
        )
        == ogr.OFTInteger
    )


def test_get_ogr_vec_col_datatype_from_gdal_rat_col_datatype_flt():
    import rsgislib.vectorutils
    from osgeo import gdal
    from osgeo import ogr

    assert (
        rsgislib.vectorutils.get_ogr_vec_col_datatype_from_gdal_rat_col_datatype(
            gdal.GFT_Real
        )
        == ogr.OFTReal
    )


def test_get_ogr_vec_col_datatype_from_gdal_rat_col_datatype_str():
    import rsgislib.vectorutils
    from osgeo import gdal
    from osgeo import ogr

    assert (
        rsgislib.vectorutils.get_ogr_vec_col_datatype_from_gdal_rat_col_datatype(
            gdal.GFT_String
        )
        == ogr.OFTString
    )


def test_copy_rat_cols_to_vector_lyr(tmp_path):
    import rsgislib.vectorutils
    import rsgislib.vectorutils.createrasters
    import rsgislib.rastergis

    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")

    vec_ref_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_file = os.path.join(tmp_path, "aber_osgb_multi_polys.geojson")
    vec_lyr = "aber_osgb_multi_polys"

    copy2(vec_ref_file, vec_file)

    clumps_img = os.path.join(tmp_path, "clumps_img.kea")

    rsgislib.vectorutils.createrasters.rasterise_vec_lyr(
        vec_file, vec_lyr, input_img, clumps_img, gdalformat="KEA", att_column="val"
    )

    bs = []
    bs.append(
        rsgislib.rastergis.BandAttStats(
            band=1, min_field="b1Min", max_field="b1Max", mean_field="b1Mean"
        )
    )
    bs.append(
        rsgislib.rastergis.BandAttStats(
            band=2, min_field="b2Min", max_field="b2Max", mean_field="b2Mean"
        )
    )
    rsgislib.rastergis.populate_rat_with_stats(input_img, clumps_img, bs)

    ratcols = ["b1Min", "b1Max", "b1Mean", "b2Min", "b2Max", "b2Mean"]
    out_col_names = ["b1Min", "b1Max", "b1Mean", "b2Min", "b2Max", "b2Mean"]

    rsgislib.vectorutils.copy_rat_cols_to_vector_lyr(
        vec_file, vec_lyr, "val", clumps_img, ratcols, out_col_names, out_col_types=None
    )


def test_vector_maths(tmp_path):
    import rsgislib.vectorutils
    import rsgislib.vectorattrs
    import numpy

    vec_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_lyr = "aber_osgb_multi_polys"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    vars = list()
    vars.append(rsgislib.vectorutils.VecColVar(name="val", field_name="val"))
    rsgislib.vectorutils.vector_maths(
        vec_file,
        vec_lyr,
        out_vec_file,
        out_vec_lyr,
        "GPKG",
        "out_vals",
        "val * val",
        vars,
        True,
    )

    # Check values are correct.
    in_vals_lst = rsgislib.vectorattrs.read_vec_column(vec_file, vec_lyr, "val")
    in_vals = numpy.array(in_vals_lst)
    calcd_out_vals = in_vals * in_vals
    out_vals_lst = rsgislib.vectorattrs.read_vec_column(
        out_vec_file, out_vec_lyr, "out_vals"
    )
    out_vals = numpy.array(out_vals_lst)
    diff = numpy.abs(calcd_out_vals - out_vals)
    total = numpy.sum(diff)
    assert os.path.exists(out_vec_file) and (total < 0.01)


def test_create_lines_of_points(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "vec_lines_lyr.geojson")
    vec_lyr = "vec_lines_lyr"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.create_lines_of_points(
        vec_file,
        vec_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        step=100,
        del_exist_vec=True,
    )
    assert os.path.exists(out_vec_file)


def test_check_validate_geometries(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_lyr = "aber_osgb_multi_polys"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.check_validate_geometries(
        vec_file,
        vec_lyr,
        out_vec_file,
        out_vec_lyr,
        "GPKG",
        print_err_geoms=True,
        del_exist_vec=True,
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(
    (GEOPANDAS_NOT_AVAIL and RTREE_NOT_AVAIL),
    reason="geopandas or rtree dependencies not available",
)
def test_perform_spatial_join_empty(tmp_path):
    import rsgislib.vectorutils

    vec_base_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_base_lyr = "aber_osgb_multi_polys"

    vec_join_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_join_lyr = "sample_pts"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.perform_spatial_join(
        vec_base_file,
        vec_base_lyr,
        vec_join_file,
        vec_join_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        join_how="inner",
        join_op="within",
    )

    assert not os.path.exists(out_vec_file)


@pytest.mark.skipif(
    (GEOPANDAS_NOT_AVAIL and RTREE_NOT_AVAIL),
    reason="geopandas or rtree dependencies not available",
)
def test_perform_spatial_join(tmp_path):
    import rsgislib.vectorutils

    vec_join_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_join_lyr = "cls_grass_smpls"

    vec_base_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_base_lyr = "sample_pts"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.perform_spatial_join(
        vec_base_file,
        vec_base_lyr,
        vec_join_file,
        vec_join_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        join_how="inner",
        join_op="within",
    )

    assert os.path.exists(out_vec_file)


def test_does_vmsk_img_intersect(tmp_path):
    import rsgislib.vectorutils

    input_vmsk_img = os.path.join(DATA_DIR, "sen2_20210527_aber_vldmsk.kea")
    vec_roi_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_roi_lyr = "aber_osgb_multi_polys"

    intersect = rsgislib.vectorutils.does_vmsk_img_intersect(
        input_vmsk_img, vec_roi_file, vec_roi_lyr, tmp_dir=tmp_path
    )
    assert intersect


def test_vector_translate(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "aber_osgb_multi_polys.geojson")
    vec_lyr = "aber_osgb_multi_polys"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.vector_translate(
        vec_file, vec_lyr, out_vec_file, out_vec_lyr, out_format="GPKG"
    )

    assert os.path.exists(out_vec_file)


def test_reproj_wgs84_vec_to_utm(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_examples_uk.geojson")
    vec_lyr = "degree_grid_examples_uk"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.reproj_wgs84_vec_to_utm(
        vec_file, vec_lyr, out_vec_file, out_vec_lyr, use_hemi=True, out_format="GPKG"
    )

    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_spatial_select(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    vec_roi_file = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_roi_lyr = "cls_grass_smpls"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.spatial_select(
        vec_file,
        vec_lyr,
        vec_roi_file,
        vec_roi_lyr,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
    )

    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_split_by_attribute_sgl_file(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    split_col_name = "names"

    rsgislib.vectorutils.split_by_attribute(
        vec_file,
        vec_lyr,
        split_col_name,
        multi_layers=True,
        out_vec_file=out_vec_file,
        out_file_path=None,
        out_file_ext="gpkg",
        out_format="GPKG",
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_split_by_attribute_mul_files(tmp_path):
    import rsgislib.vectorutils
    import glob

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    split_col_name = "names"

    rsgislib.vectorutils.split_by_attribute(
        vec_file,
        vec_lyr,
        split_col_name,
        multi_layers=False,
        out_vec_file=None,
        out_file_path=tmp_path,
        out_file_ext="gpkg",
        out_format="GPKG",
    )
    assert len(glob.glob(os.path.join(tmp_path, "*.gpkg"))) == 13


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_subset_by_attribute_equals(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    sub_col = "names"
    sub_vals = ["helloN63W125world", "helloN63W123world", "helloS52E053world"]
    rsgislib.vectorutils.subset_by_attribute(
        vec_file,
        vec_lyr,
        sub_col,
        sub_vals,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        match_type="equals",
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_subset_by_attribute_contains(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    sub_col = "names"
    sub_vals = ["N63"]
    rsgislib.vectorutils.subset_by_attribute(
        vec_file,
        vec_lyr,
        sub_col,
        sub_vals,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        match_type="contains",
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_subset_by_attribute_start(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    sub_col = "names"
    sub_vals = ["hello"]
    rsgislib.vectorutils.subset_by_attribute(
        vec_file,
        vec_lyr,
        sub_col,
        sub_vals,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
        match_type="start",
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_drop_rows_by_attribute(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_named_subset.geojson")
    vec_lyr = "degree_grid_named_subset"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    sub_col = "names"
    sub_vals = ["N63"]
    rsgislib.vectorutils.drop_rows_by_attribute(
        vec_file,
        vec_lyr,
        sub_col,
        sub_vals,
        out_vec_file,
        out_vec_lyr,
        out_format="GPKG",
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_merge_vector_files(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = [vec_file_1, vec_file_2, vec_file_3, vec_file_4]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.merge_vector_files(
        in_vec_files, out_vec_file, out_vec_lyr, out_format="GPKG"
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_merge_vector_layers(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "cls_forest_smpls.gpkg")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "cls_grass_smpls.gpkg")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "cls_urban_smpls.gpkg")
    vec_file_4 = os.path.join(VECTORUTILS_DATA_DIR, "cls_water_smpls.gpkg")

    in_vec_files = list()
    in_vec_files.append({"file": vec_file_1, "layer": "cls_forest_smpls"})
    in_vec_files.append({"file": vec_file_2, "layer": "cls_grass_smpls"})
    in_vec_files.append({"file": vec_file_3, "layer": "cls_urban_smpls"})
    in_vec_files.append({"file": vec_file_4, "layer": "cls_water_smpls"})

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"
    rsgislib.vectorutils.merge_vector_layers(
        in_vec_files, out_vec_file, out_vec_lyr, out_format="GPKG"
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(
    (GEOPANDAS_NOT_AVAIL and SHAPELY_NOT_AVAIL),
    reason="geopandas or shapely dependencies not available",
)
def test_geopd_check_polys_wgs84_bounds_geometry():
    import rsgislib.vectorutils
    import geopandas

    vec_file = os.path.join(VECTORUTILS_DATA_DIR, "fiji_wgs84_poly.geojson")
    vec_lyr = "fiji_wgs84_poly"

    data_gdf = geopandas.read_file(vec_file, layer=vec_lyr)

    gdf_out = rsgislib.vectorutils.geopd_check_polys_wgs84_bounds_geometry(data_gdf)
    assert len(gdf_out) > 0


# TODO - issues on windows so skipping for the moment.
@pytest.mark.skipif(
    on_windows or (GEOPANDAS_NOT_AVAIL and SHAPELY_NOT_AVAIL),
    reason="geopandas or shapely dependencies not available or skipping on Windows",
)
def test_merge_utm_vecs_wgs84(tmp_path):
    import rsgislib.vectorutils

    vec_file_1 = os.path.join(VECTORUTILS_DATA_DIR, "utm_1n_polys.geojson")
    vec_file_2 = os.path.join(VECTORUTILS_DATA_DIR, "utm_30n_polys.geojson")
    vec_file_3 = os.path.join(VECTORUTILS_DATA_DIR, "utm_60s_polys.geojson")
    in_vec_files = [vec_file_1, vec_file_2, vec_file_3]

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.merge_utm_vecs_wgs84(
        in_vec_files, out_vec_file, out_vec_lyr, out_format="GPKG"
    )
    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_split_feats_to_mlyrs(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(DATA_DIR, "degree_grid_examples_uk.geojson")
    vec_lyr = "degree_grid_examples_uk"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")

    rsgislib.vectorutils.split_feats_to_mlyrs(
        vec_file, vec_lyr, out_vec_file, out_format="GPKG"
    )

    assert os.path.exists(out_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_split_vec_lyr_random_subset(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    out_rmain_vec_file = os.path.join(tmp_path, "remain_out_vec.gpkg")
    out_rmain_vec_lyr = "remain_out_vec"

    out_smpl_vec_file = os.path.join(tmp_path, "sample_out_vec.gpkg")
    out_smpl_vec_lyr = "sample_out_vec"

    rsgislib.vectorutils.split_vec_lyr_random_subset(
        vec_file,
        vec_lyr,
        out_rmain_vec_file,
        out_rmain_vec_lyr,
        out_smpl_vec_file,
        out_smpl_vec_lyr,
        n_smpl=10,
        out_format="GPKG",
        rnd_seed=42,
    )

    assert os.path.exists(out_rmain_vec_file) and os.path.exists(out_smpl_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_create_train_test_smpls(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(REGRESS_DATA_DIR, "sample_pts.geojson")
    vec_lyr = "sample_pts"

    out_train_vec_file = os.path.join(tmp_path, "train_out_vec.gpkg")
    out_train_vec_lyr = "train_out_vec"

    out_test_vec_file = os.path.join(tmp_path, "test_out_vec.gpkg")
    out_test_vec_lyr = "sample_out_vec"

    rsgislib.vectorutils.create_train_test_smpls(
        vec_file,
        vec_lyr,
        out_train_vec_file,
        out_train_vec_lyr,
        out_test_vec_file,
        out_test_vec_lyr,
        out_format="GPKG",
        prop_test=0.2,
        tmp_dir=tmp_path,
        rnd_seed=42,
    )
    assert os.path.exists(out_train_vec_file) and os.path.exists(out_test_vec_file)


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_rm_feat_att_duplicates_str(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(
        VECTORUTILS_DATA_DIR, "sen2_20210527_aber_att_vals_dups.geojson"
    )
    vec_lyr = "sen2_20210527_aber_att_vals_dups"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.rm_feat_att_duplicates(
        vec_file, vec_lyr, "StrCol", out_vec_file, out_vec_lyr, out_format="GPKG"
    )

    n_feats = rsgislib.vectorutils.get_vec_feat_count(out_vec_file, out_vec_lyr)
    assert n_feats == 3


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_rm_feat_att_duplicates_int(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(
        VECTORUTILS_DATA_DIR, "sen2_20210527_aber_att_vals_dups.geojson"
    )
    vec_lyr = "sen2_20210527_aber_att_vals_dups"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.rm_feat_att_duplicates(
        vec_file, vec_lyr, "IntCol", out_vec_file, out_vec_lyr, out_format="GPKG"
    )

    n_feats = rsgislib.vectorutils.get_vec_feat_count(out_vec_file, out_vec_lyr)
    assert n_feats == 3


@pytest.mark.skipif(GEOPANDAS_NOT_AVAIL, reason="geopandas dependency not available")
def test_rm_feat_att_duplicates_float(tmp_path):
    import rsgislib.vectorutils

    vec_file = os.path.join(
        VECTORUTILS_DATA_DIR, "sen2_20210527_aber_att_vals_dups.geojson"
    )
    vec_lyr = "sen2_20210527_aber_att_vals_dups"

    out_vec_file = os.path.join(tmp_path, "out_vec.gpkg")
    out_vec_lyr = "out_vec"

    rsgislib.vectorutils.rm_feat_att_duplicates(
        vec_file, vec_lyr, "FloatCol", out_vec_file, out_vec_lyr, out_format="GPKG"
    )

    n_feats = rsgislib.vectorutils.get_vec_feat_count(out_vec_file, out_vec_lyr)
    assert n_feats == 3



#!/usr/bin/env python
"""
Contains image filtering functionality for rsgislib
"""

# import the C++ extension into this level
from ._imagefilter import *
import os


class FilterParameters:
    """Object, specifying the type of filter and filter parameters"""

    def __init__(
        self,
        filter_type,
        file_ending,
        size=3,
        option=None,
        n_looks=None,
        stddev=None,
        stddev_x=None,
        stddev_y=None,
        angle=None,
    ):
        self.filter_type = filter_type
        self.file_ending = file_ending
        self.size = size
        self.option = option
        self.n_looks = n_looks
        self.stddev = stddev
        self.stddev_x = stddev_x
        self.stddev_y = stddev_y
        self.angle = angle


def apply_median_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a median filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_median3.kea'
        imagefilter.apply_median_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Median", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_mean_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a mean filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_mean3.kea'
        imagefilter.apply_mean_filter(input_img, outImgFile, 3, "KEA",
                                    rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Mean", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_min_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a min filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_min3.kea'
        imagefilter.apply_min_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Min", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_max_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a max filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_max3.kea'
        imagefilter.apply_max_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Max", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_mode_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a mode filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_mode3.kea'
        imagefilter.apply_mode_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Mode", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_stddev_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a std dev filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_stdev3.kea'
        imagefilter.apply_stddev_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="StdDev", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_range_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a range filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_range3.kea'
        imagefilter.apply_range_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Range", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_mean_diff_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a mean difference filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_meandiff3.kea'
        imagefilter.apply_mean_diff_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="MeanDiff", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_mean_diff_abs_filter(
    input_img, output_img, filter_size, gdalformat, datatype
):
    """
    Apply a mean absolute difference filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_meandiffabs3.kea'
        imagefilter.apply_mean_diff_abs_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="MeanDiffAbs", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_total_diff_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a total (i.e., sum) difference filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_totaldiff3.kea'
        imagefilter.apply_total_diff_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="TotalDiff", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_total_diff_abs_filter(
    input_img, output_img, filter_size, gdalformat, datatype
):
    """
    Apply a total absolute difference filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_totaldiffabs3.kea'
        imagefilter.apply_total_diff_abs_filter(input_img, outImgFile, 3, "KEA",
                                                rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="TotalDiffAbs", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_coeff_of_var_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a coefficient of variance filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_cofvar3.kea'
        imagefilter.apply_coeff_of_var_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="CoeffOfVar", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_total_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a total filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_total3.kea'
        imagefilter.apply_total_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Total", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_norm_var_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a normalised variance filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_NormVar3.kea'
        imagefilter.apply_norm_var_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="NormVar", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_norm_var_sqrt_filter(
    input_img, output_img, filter_size, gdalformat, datatype
):
    """
    Apply a normalised variance square root filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_NormVarSqrt3.kea'
        imagefilter.apply_norm_var_sqrt_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="NormVarSqrt", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_norm_var_ln_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a normalised variance natural log filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_NormVarLn3.kea'
        imagefilter.apply_norm_var_ln_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="NormVarLn", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_texture_var_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a texture variance filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_NormVarLn3.kea'
        imagefilter.apply_texture_var_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="TextureVar", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_kuwahara_filter(input_img, output_img, filter_size, gdalformat, datatype):
    """
    Apply a kuwahara filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter
                        (must be an odd number, i.e., 3, 5, 7, etc).
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_kuwa3.kea'
        imagefilter.apply_kuwahara_filter(input_img, outImgFile, 3, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(filter_type="Kuwahara", file_ending="", size=filter_size)
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_sobel_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a sobel filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_sobel.kea'
        imagefilter.apply_sobel_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Sobel", file_ending="", option="xy"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_sobel_x_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a sobel filter in X axis to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_sobelx.kea'
        imagefilter.apply_sobel_x_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Sobel", file_ending="", option="x"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_sobel_y_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a sobel filter in Y axis to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_sobely.kea'
        imagefilter.apply_sobel_y_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Sobel", file_ending="", option="y"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_prewitt_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a Prewitt filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_prewitt.kea'
        imagefilter.apply_prewitt_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Prewitt", file_ending="", option="xy"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_prewitt_x_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a Prewitt filter in X axis to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_prewitt.kea'
        imagefilter.apply_prewitt_x_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Prewitt", file_ending="", option="x"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_prewitt_y_filter(input_img, output_img, gdalformat, datatype):
    """
    Apply a Prewitt filter in Y axis to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_prewitt.kea'
        imagefilter.apply_prewitt_y_filter(input_img, outImgFile, "KEA",
                                      rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(FilterParameters(filter_type="Prewitt", file_ending="", option="y"))
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_gaussian_smooth_filter(
    input_img,
    output_img,
    filter_size,
    stddev_x,
    stddev_y,
    filter_angle,
    gdalformat,
    datatype,
):
    """
    Apply a Gaussian smoothing filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter (must be an
                        odd number, i.e., 3, 5, 7, etc).
    :param stddev_x: standard deviation in the x-axis
    :param stddev_y: standard deviation in the y-axis
    :param filter_angle: the angle of the filter
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_gausmooth.kea'
        imagefilter.apply_gaussian_smooth_filter(input_img, outImgFile, 3, 1.0, 1.0. 0.0,
                                              "KEA", rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(
            filter_type="GaussianSmooth",
            file_ending="",
            size=filter_size,
            stddev_x=stddev_x,
            stddev_y=stddev_y,
            angle=filter_angle,
        )
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_gaussian_1st_deriv_filter(
    input_img,
    output_img,
    filter_size,
    stddev_x,
    stddev_y,
    filter_angle,
    gdalformat,
    datatype,
):
    """
    Apply a Gaussian first derivative filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter (must be an
                        odd number, i.e., 3, 5, 7, etc).
    :param stddev_x: standard deviation in the x-axis
    :param stddev_y: standard deviation in the y-axis
    :param filter_angle: the angle of the filter
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_gau1st.kea'
        imagefilter.apply_gaussian_1st_deriv_filter(input_img, outImgFile, 3, 1.0, 1.0. 0.0,
                                                "KEA", rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(
            filter_type="Gaussian1st",
            file_ending="",
            size=filter_size,
            stddev_x=stddev_x,
            stddev_y=stddev_y,
            angle=filter_angle,
        )
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_gaussian_2nd_deriv_filter(
    input_img,
    output_img,
    filter_size,
    stddev_x,
    stddev_y,
    filter_angle,
    gdalformat,
    datatype,
):
    """
    Apply a Gaussian second derivative filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter (must be an
                        odd number, i.e., 3, 5, 7, etc).
    :param stddev_x: standard deviation in the x-axis
    :param stddev_y: standard deviation in the y-axis
    :param filter_angle: the angle of the filter
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_gau1st.kea'
        imagefilter.apply_gaussian_2nd_deriv_filter(input_img, outImgFile, 3, 1.0,
                                                    1.0. 0.0, "KEA",
                                                    rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(
            filter_type="Gaussian2nd",
            file_ending="",
            size=filter_size,
            stddev_x=stddev_x,
            stddev_y=stddev_y,
            angle=filter_angle,
        )
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_laplacian_filter(
    input_img, output_img, filter_size, stddev, gdalformat, datatype
):
    """
    Apply a Laplacian filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter (must be an
                        odd number, i.e., 3, 5, 7, etc).
    :param stddev: standard deviation of the filter
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_laplacian.kea'
        imagefilter.apply_laplacian_filter(input_img, outImgFile, 3, 1.0, "KEA",
                                           rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(
            filter_type="Laplacian", file_ending="", size=filter_size, stddev=stddev
        )
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)


def apply_lee_filter(input_img, output_img, filter_size, n_looks, gdalformat, datatype):
    """Apply a Lee SAR filter to the specified input image.

    :param input_img: string specifying the input image to be filtered.
    :param output_img: string specifying the output image file..
    :param filter_size: int specifying the size of the image filter (must be an
                        odd number, i.e., 3, 5, 7, etc).
    :param n_looks: int specifying the number of looks applied to the SAR image.
    :param gdalformat: string specifying the output image format (e.g., KEA).
    :param datatype: Specifying the output image pixel data type
                     (e.g., rsgislib.TYPE_32FLOAT).

    .. code:: python

        import rsgislib
        from rsgislib import imagefilter
        input_img = 'jers1palsar_stack.kea'
        outImgFile = 'jers1palsar_stack_lee.kea'
        imagefilter.apply_lee_filter(input_img, outImgFile, 3, 3, "KEA",
                                     rsgislib.TYPE_32FLOAT)

    """
    outputImageBase, outExt = os.path.splitext(output_img)
    outExt = outExt.replace(".", "").strip()
    filters = []
    filters.append(
        FilterParameters(
            filter_type="Lee", file_ending="", size=filter_size, n_looks=n_looks
        )
    )
    apply_filters(input_img, outputImageBase, filters, gdalformat, outExt, datatype)



#!/usr/bin/env python
"""
Tools for developing and applying regression models.

"""

import rsgislib


def get_regression_stats(ref_data, pred_data, n_vars=1):
    """
    A function which calculates a set of accuracy metrics using a set of
    reference and predicted values.

    if n_vars == 1, then ref_data and pred_data can be flat arrays.

    :param ref_data: a numpy array of n x m (m = n_vars) with the reference values.
    :param pred_data: a numpy array of n x m (m = n_vars) with the predicted values.
    :param n_vars: the number of variables to be used.
    :return: list of dicts

    """
    import sklearn.metrics
    import numpy
    import rsgislib.tools.stats

    ref_data_shp = ref_data.shape
    pred_data_shp = ref_data.shape

    if ref_data_shp[0] != pred_data_shp[0]:
        raise rsgislib.RSGISPyException(
            "The length of the reference and predicted arrays must be the same."
        )

    if n_vars == 1:
        if len(ref_data_shp) == 1:
            ref_data = numpy.expand_dims(ref_data, axis=1)
        if len(pred_data_shp) == 1:
            pred_data = numpy.expand_dims(pred_data, axis=1)

    ref_data_shp = ref_data.shape
    pred_data_shp = ref_data.shape

    if ref_data_shp[1] != pred_data_shp[1]:
        raise rsgislib.RSGISPyException(
            "The number of variables for the reference and predicted arrays must be the same."
        )

    if ref_data_shp[1] != n_vars:
        raise rsgislib.RSGISPyException(
            "The number of variables for the reference and predicted arrays must be the same as the number of variables specified in n_vars"
        )

    out_acc_metrics = list()
    for i in range(n_vars):
        acc_metrics = dict()
        acc_metrics["r2"] = sklearn.metrics.r2_score(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics[
            "explained_variance_score"
        ] = sklearn.metrics.explained_variance_score(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics["median_absolute_error"] = sklearn.metrics.median_absolute_error(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics["mean_absolute_error"] = sklearn.metrics.mean_absolute_error(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics["mean_squared_error"] = sklearn.metrics.mean_squared_error(
            ref_data[..., i], pred_data[..., i]
        )
        rmse = numpy.sqrt(
            sklearn.metrics.mean_squared_error(ref_data[..., i], pred_data[..., i])
        )
        acc_metrics["root_mean_squared_error"] = rmse
        acc_metrics["norm_root_mean_squared_error"] = 100 * (
            rmse / numpy.mean(ref_data[..., i])
        )
        bias, norm_bias = rsgislib.tools.stats.bias_score(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics["bias"] = bias
        acc_metrics["norm_bias"] = norm_bias
        (
            mse,
            bias_squared,
            variance,
            noise,
        ) = rsgislib.tools.stats.decompose_bias_variance(
            ref_data[..., i], pred_data[..., i]
        )
        acc_metrics["bias_squared"] = bias_squared
        acc_metrics["variance"] = variance
        acc_metrics["noise"] = noise
        out_acc_metrics.append(acc_metrics)

    return out_acc_metrics



def test_reproj_point_OSGB_UTM():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromProj4(
        "+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 "
        "+x_0=400000 +y_0=-100000 +ellps=airy +datum=OSGB36 "
        "+units=m +no_defs"
    )

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(32630)

    osgb_x = 258051.0
    osgb_y = 281605.0

    utm_x, utm_y = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, osgb_x, osgb_y
    )

    ref_utm_x = 425966.1
    ref_utm_y = 5807642.0
    assert (abs(ref_utm_x - utm_x) < 2) and (abs(ref_utm_y - utm_y) < 2)


def test_reproj_point_OSGB_WGS84():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromProj4(
        "+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 "
        "+x_0=400000 +y_0=-100000 +ellps=airy +datum=OSGB36 "
        "+units=m +no_defs"
    )

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(4326)

    osgb_x = 258051.0
    osgb_y = 281605.0

    lon, lat = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, osgb_x, osgb_y
    )

    ref_lon = -4.0885
    ref_lat = 52.4140
    assert (abs(ref_lon - lon) < 0.001) and (abs(ref_lat - lat) < 0.001)


def test_reproj_point_WGS84_OSGB():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromEPSG(4326)

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromProj4(
        "+proj=tmerc +lat_0=49 +lon_0=-2 +k=0.9996012717 "
        "+x_0=400000 +y_0=-100000 +ellps=airy +datum=OSGB36 "
        "+units=m +no_defs"
    )

    lon = -4.0885
    lat = 52.4140

    osgb_x, osgb_y = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, lon, lat
    )

    ref_osgb_x = 258051.3
    ref_osgb_y = 281606.4
    assert (abs(ref_osgb_x - osgb_x) < 2) and (abs(ref_osgb_y - osgb_y) < 2)


def test_reproj_point_WGS84_UTM():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromEPSG(4326)

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(32630)

    lon = -4.0885
    lat = 52.4140

    utm_x, utm_y = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, lon, lat
    )

    ref_utm_x = 425966.3
    ref_utm_y = 5807643.4
    assert (abs(ref_utm_x - utm_x) < 0.1) and (abs(ref_utm_y - utm_y) < 0.1)


def test_reproj_point_UTM_WGS84():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromEPSG(32630)

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(4326)

    utm_x = 425966.3
    utm_y = 5807643.4

    lon, lat = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, utm_x, utm_y
    )

    ref_lon = -4.0885
    ref_lat = 52.4140
    assert (abs(ref_lon - lon) < 0.001) and (abs(ref_lat - lat) < 0.001)


def test_reproj_point_NZTM_WGS84():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromProj4(
        "+proj=tmerc +lat_0=0 +lon_0=173 +k=0.9996 "
        "+x_0=1600000 +y_0=10000000 +ellps=GRS80 "
        "+towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
    )

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(4326)

    nztm_x = 1822776.0
    nztm_y = 5527532.6

    lon, lat = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, nztm_x, nztm_y
    )

    ref_lon = 175.6240
    ref_lat = -40.3730

    assert (abs(ref_lat - lat) < 0.001) and (abs(ref_lon - lon) < 0.001)


def test_reproj_point_NZTM_UTM():
    import rsgislib.tools.geometrytools
    from osgeo import osr

    in_osr_proj_obj = osr.SpatialReference()
    in_osr_proj_obj.ImportFromProj4(
        "+proj=tmerc +lat_0=0 +lon_0=173 +k=0.9996 "
        "+x_0=1600000 +y_0=10000000 +ellps=GRS80 "
        "+towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
    )

    out_osr_proj_obj = osr.SpatialReference()
    out_osr_proj_obj.ImportFromEPSG(32760)

    nztm_x = 1822776.0
    nztm_y = 5527532.6

    utm_x, utm_y = rsgislib.tools.geometrytools.reproj_point(
        in_osr_proj_obj, out_osr_proj_obj, nztm_x, nztm_y
    )

    ref_utm_x = 383187.9
    ref_utm_y = 5529929.6
    assert (abs(ref_utm_x - utm_x) < 0.1) and (abs(ref_utm_y - utm_y) < 0.1)



import os
import platform
import pytest
import rsgislib.tools.filetools

DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
FILETOOLS_DATA_DIR = os.path.join(DATA_DIR, "tools/filetools")

unzip_cmd_avail = rsgislib.tools.filetools.is_cmd_tool_avail("unzip")
tar_cmd_avail = rsgislib.tools.filetools.is_cmd_tool_avail("tar")
on_windows = platform.system() == "Windows"


@pytest.mark.parametrize(
    "input, expected",
    [("hello.txt", "hello"), ("world/hello.txt", "hello"), ("hello.tar.gz", "hello")],
)
def test_get_file_basename(input, expected):
    assert rsgislib.tools.filetools.get_file_basename(input) == expected


def test_get_file_basename_rm_n_exts_1():
    assert (
        rsgislib.tools.filetools.get_file_basename("hello.tar.gz", rm_n_exts=1)
        == "hello.tar"
    )


def test_get_file_basename_check_valid():
    assert (
        rsgislib.tools.filetools.get_file_basename("hello!.txt", check_valid=True)
        == "hello"
    )


def test_get_file_basename_check_valid_1():
    assert (
        rsgislib.tools.filetools.get_file_basename(
            "hello_world_rsgislib.txt", n_comps=1
        )
        == "hello"
    )


def test_get_file_basename_check_valid_2():
    assert (
        rsgislib.tools.filetools.get_file_basename(
            "hello_world_rsgislib.txt", n_comps=2
        )
        == "hello_world"
    )


def test_is_path_valid_true():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert rsgislib.tools.filetools.is_path_valid(input_img)


def test_is_path_valid_false(tmp_path):
    input_file = os.path.join(tmp_path, "sen2_20210527_aber\x00.kea")
    assert not rsgislib.tools.filetools.is_path_valid(input_file)


def test_is_path_sibling_creatable_true(tmp_path):
    input_file = os.path.join(tmp_path, "sen2_20210527_aber.kea")
    assert rsgislib.tools.filetools.is_path_sibling_creatable(input_file)


def test_is_path_sibling_creatable_false(tmp_path):
    input_file = os.path.join(tmp_path, "hello", "world", "sen2_20210527_aber.kea")
    assert not rsgislib.tools.filetools.is_path_sibling_creatable(input_file)


def test_does_path_exists_or_creatable_true(tmp_path):
    input_file = os.path.join(tmp_path, "sen2_20210527_aber.kea")
    assert rsgislib.tools.filetools.does_path_exists_or_creatable(input_file)


def test_does_path_exists_or_creatable_false_file_name(tmp_path):
    input_file = os.path.join(tmp_path, "sen2_20210527_aber\x00.kea")
    assert not rsgislib.tools.filetools.does_path_exists_or_creatable(input_file)


def test_does_path_exists_or_creatable_false_path(tmp_path):
    input_file = os.path.join(tmp_path, "hello", "world", "sen2_20210527_aber.kea")
    assert not rsgislib.tools.filetools.does_path_exists_or_creatable(input_file)


def test_does_path_exists_or_creatable_false_path_and_name(tmp_path):
    input_file = os.path.join(tmp_path, "hello", "world", "sen2_20210527_aber\x00.kea")
    assert not rsgislib.tools.filetools.does_path_exists_or_creatable(input_file)


def test_get_file_size_bytes():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert rsgislib.tools.filetools.get_file_size(input_img) == 7749324


def test_get_file_size_kb():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert (
        abs(rsgislib.tools.filetools.get_file_size(input_img, unit="kb") - 7567.699)
        < 0.1
    )


def test_get_file_size_mb():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert (
        abs(rsgislib.tools.filetools.get_file_size(input_img, unit="mb") - 7.39) < 0.01
    )


def test_get_file_size_gb():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert (
        abs(rsgislib.tools.filetools.get_file_size(input_img, unit="gb") - 0.00722)
        < 0.0001
    )


def test_get_file_size_tb():
    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    assert (
        abs(rsgislib.tools.filetools.get_file_size(input_img, unit="tb") - 7.04797e-06)
        < 0.0000001
    )


def test_convert_file_size_units_bytes_kb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000000, in_unit="bytes", out_unit="kb"
    )
    assert abs(out_size - 976.56) < 0.1


def test_convert_file_size_units_bytes_mb():
    import rsgislib.tools.filetools

    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000000, in_unit="bytes", out_unit="mb"
    )
    assert abs(out_size - 0.954) < 0.1


def test_convert_file_size_units_bytes_gb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000000, in_unit="bytes", out_unit="gb"
    )
    assert abs(out_size - 0.000931) < 0.0001


def test_convert_file_size_units_bytes_tb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000000, in_unit="bytes", out_unit="tb"
    )
    assert abs(out_size - 9.095e-07) < 0.0000001


def test_convert_file_size_units_kb_bytes():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000, in_unit="kb", out_unit="bytes"
    )
    assert abs(out_size - 1024000.0) < 1


def test_convert_file_size_units_kb_mb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000, in_unit="kb", out_unit="mb"
    )
    assert abs(out_size - 0.977) < 0.1


def test_convert_file_size_units_kb_gb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000, in_unit="kb", out_unit="gb"
    )
    assert abs(out_size - 0.000954) < 0.0001


def test_convert_file_size_units_kb_tb():
    out_size = rsgislib.tools.filetools.convert_file_size_units(
        1000, in_unit="kb", out_unit="tb"
    )
    assert abs(out_size - 9.313e-07) < 0.0000001


@pytest.mark.skipif((not tar_cmd_avail), reason="tar command not available")
def test_untar_file_gen_arch_dir_true(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar")

    rsgislib.tools.filetools.untar_file(input_file, tmp_path)

    hello_file = os.path.join(tmp_path, "test_file", "hello.txt")
    world_file = os.path.join(tmp_path, "test_file", "world.txt")
    rsgislib_file = os.path.join(tmp_path, "test_file", "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif((not tar_cmd_avail), reason="tar command not available")
def test_untar_file_gen_arch_dir_false(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar")

    rsgislib.tools.filetools.untar_file(input_file, tmp_path, gen_arch_dir=False)

    hello_file = os.path.join(tmp_path, "hello.txt")
    world_file = os.path.join(tmp_path, "world.txt")
    rsgislib_file = os.path.join(tmp_path, "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif((not tar_cmd_avail), reason="tar command not available")
def test_untar_gz_file_gen_arch_dir_true(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar.gz")

    rsgislib.tools.filetools.untar_gz_file(input_file, tmp_path)

    hello_file = os.path.join(tmp_path, "test_file", "hello.txt")
    world_file = os.path.join(tmp_path, "test_file", "world.txt")
    rsgislib_file = os.path.join(tmp_path, "test_file", "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif((not tar_cmd_avail), reason="tar command not available")
def test_untar_gz_file_gen_arch_dir_false(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar.gz")

    rsgislib.tools.filetools.untar_gz_file(input_file, tmp_path, gen_arch_dir=False)

    hello_file = os.path.join(tmp_path, "hello.txt")
    world_file = os.path.join(tmp_path, "world.txt")
    rsgislib_file = os.path.join(tmp_path, "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif(
    on_windows or (not tar_cmd_avail), reason="tar command not available or on Windows"
)
def test_untar_bz_file_gen_arch_dir_true(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar.bz")

    rsgislib.tools.filetools.untar_bz_file(input_file, tmp_path)

    hello_file = os.path.join(tmp_path, "test_file", "hello.txt")
    world_file = os.path.join(tmp_path, "test_file", "world.txt")
    rsgislib_file = os.path.join(tmp_path, "test_file", "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif(
    on_windows or (not tar_cmd_avail), reason="tar command not available or on Windows"
)
def test_untar_bz_file_gen_arch_dir_false(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.tar.bz")

    rsgislib.tools.filetools.untar_bz_file(input_file, tmp_path, gen_arch_dir=False)

    hello_file = os.path.join(tmp_path, "hello.txt")
    world_file = os.path.join(tmp_path, "world.txt")
    rsgislib_file = os.path.join(tmp_path, "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif((not unzip_cmd_avail), reason="unzip command not available")
def test_unzip_file_gen_arch_dir_true(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.zip")

    rsgislib.tools.filetools.unzip_file(input_file, tmp_path)

    hello_file = os.path.join(tmp_path, "test_file", "hello.txt")
    world_file = os.path.join(tmp_path, "test_file", "world.txt")
    rsgislib_file = os.path.join(tmp_path, "test_file", "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


@pytest.mark.skipif((not unzip_cmd_avail), reason="unzip command not available")
def test_unzip_file_gen_arch_dir_false(tmp_path):
    input_file = os.path.join(FILETOOLS_DATA_DIR, "test_file.zip")

    rsgislib.tools.filetools.unzip_file(input_file, tmp_path, gen_arch_dir=False)

    hello_file = os.path.join(tmp_path, "hello.txt")
    world_file = os.path.join(tmp_path, "world.txt")
    rsgislib_file = os.path.join(tmp_path, "rsgislib.txt")

    success = (
        os.path.exists(hello_file)
        and os.path.exists(world_file)
        and os.path.exists(rsgislib_file)
    )
    assert success


def test_delete_file_silent(tmp_path):
    import pathlib

    file_to_del = os.path.join(tmp_path, "hello.txt")

    # Create a file to be deleted.
    pathlib.Path(file_to_del).touch()

    if not os.path.exists(file_to_del):
        success = False  # Something went wrong and file wasn't created.
    else:
        rsgislib.tools.filetools.delete_file_silent(file_to_del)
        # File should no longer exist
        success = not os.path.exists(file_to_del)
    assert success


def test_delete_file_with_basename(tmp_path):
    import pathlib

    # Create some file names where the base is the same but extension is different
    file_1_to_del = os.path.join(tmp_path, "hello.shp")
    file_2_to_del = os.path.join(tmp_path, "hello.shx")
    file_3_to_del = os.path.join(tmp_path, "hello.txt")

    # Create files to be deleted.
    pathlib.Path(file_1_to_del).touch()
    pathlib.Path(file_2_to_del).touch()
    pathlib.Path(file_3_to_del).touch()

    if (
        (not os.path.exists(file_1_to_del))
        or (not os.path.exists(file_2_to_del))
        or (not os.path.exists(file_3_to_del))
    ):
        success = False  # Something went wrong and file wasn't created.
    else:
        rsgislib.tools.filetools.delete_file_with_basename(file_1_to_del)
        # Files should no longer exist
        success = (
            (not os.path.exists(file_1_to_del))
            and (not os.path.exists(file_2_to_del))
            and (not os.path.exists(file_3_to_del))
        )
    assert success


def test_split_path_all_abs():
    import rsgislib.tools.filetools

    parts = rsgislib.tools.filetools.split_path_all("/a/b/c/d")
    assert (
        (parts[0] == "/")
        and (parts[1] == "a")
        and (parts[2] == "b")
        and (parts[3] == "c")
        and (parts[4] == "d")
    )


def test_split_path_all_rel():
    import rsgislib.tools.filetools

    parts = rsgislib.tools.filetools.split_path_all("a/b/c/d")
    assert (
        (parts[0] == "a")
        and (parts[1] == "b")
        and (parts[2] == "c")
        and (parts[3] == "d")
    )


def test_split_path_all_file_abs():
    import rsgislib.tools.filetools

    parts = rsgislib.tools.filetools.split_path_all("/a/b/c/hello.txt")
    assert (
        (parts[0] == "/")
        and (parts[1] == "a")
        and (parts[2] == "b")
        and (parts[3] == "c")
        and (parts[4] == "hello.txt")
    )


def test_split_path_all_file_rel():
    import rsgislib.tools.filetools

    parts = rsgislib.tools.filetools.split_path_all("a/b/c/hello.txt")
    assert (
        (parts[0] == "a")
        and (parts[1] == "b")
        and (parts[2] == "c")
        and (parts[3] == "hello.txt")
    )


def test_get_dir_name_file():
    import rsgislib.tools.filetools

    input_img = os.path.join(DATA_DIR, "sen2_20210527_aber.kea")
    dir_name = rsgislib.tools.filetools.get_dir_name(input_img)
    assert dir_name == "data"


def test_get_dir_name_dir():
    import rsgislib.tools.filetools

    dir_name = rsgislib.tools.filetools.get_dir_name(DATA_DIR)
    assert dir_name == "data"
